#include "NGTP-IF1-DispatchingServices.h"
#include "rtxsrc/rtxCommon.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"

#include <stdio.h>
#include <stdlib.h>

#define MAXREADLEN 1024

int main (int argc, char** argv)
{
   OSBOOL       aligned = TRUE, trace = TRUE, verbose = FALSE;
   const char*  filename = "message.dat";
   int          i, stat;

   // Process command line arguments
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!OSCRTLSTRCMP (argv[i], "-v")) verbose = TRUE;
         else if (!OSCRTLSTRCMP (argv[i], "-i")) filename = argv[++i];
         else if (!OSCRTLSTRCMP (argv[i], "-a")) aligned = TRUE;
         else if (!OSCRTLSTRCMP (argv[i], "-u")) aligned = FALSE;
         else if (!OSCRTLSTRCMP (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [-v] [-i <filename>] [-a|-u] [-notrace]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -notrace  do not display trace info\n");
            return 1;
         }
      }
   }

   ASN1PERDecodeBuffer decodeBuffer (aligned);
   ASN1T_DispatchingUplinkMessage data;
   ASN1C_DispatchingUplinkMessage uplinkPDU (decodeBuffer, data);

   decodeBuffer.setDiag (verbose);
   decodeBuffer.readBinaryFile (filename);
   if (decodeBuffer.getStatus() != 0) {
      printf("Error opening %s for read access\n", filename);
      decodeBuffer.printErrorInfo();
      return -1;
   }
   decodeBuffer.SetTrace (trace);

   stat = uplinkPDU.Decode ();
   if (stat != 0) {
      printf ("decode of DispatchingUplinkMessage failed\n");
      decodeBuffer.PrintErrorInfo ();
      return stat;
   }

   if (trace) {
      printf ("Dump of decoded bit fields:\n");
      decodeBuffer.BinDump ("Data");
      printf ("\n");
      printf ("Decode of data was successful\n");
      printf ("Decoded record:\n");
      uplinkPDU.Print ("DispatchingUplinkMessage");
   }
   return 0;
}
